@@ -1,6 +1,4 @@
-/*  $Id$
-
-    Creator: make/0
+/*  Creator: make/0
 
     Purpose: Provide index for autoload
 */
@@ -61,6 +59,14 @@ index((maplist), 2, apply, apply).
 index((maplist), 3, apply, apply).
 index((maplist), 4, apply, apply).
 index((maplist), 5, apply, apply).
+index((foldl), 4, apply, apply).
+index((foldl), 5, apply, apply).
+index((foldl), 6, apply, apply).
+index((foldl), 7, apply, apply).
+index((scanl), 4, apply, apply).
+index((scanl), 5, apply, apply).
+index((scanl), 6, apply, apply).
+index((scanl), 7, apply, apply).
 index((arithmetic_function), 1, arithmetic, arithmetic).
 index((arithmetic_expression_value), 2, arithmetic, arithmetic).
 index((empty_assoc), 1, assoc, assoc).
@@ -85,6 +91,7 @@ index(('$arch'), 2, backward_compatibility, backcomp).
 index(('$version'), 1, backward_compatibility, backcomp).
 index(('$home'), 1, backward_compatibility, backcomp).
 index(('$argv'), 1, backward_compatibility, backcomp).
+index(('$set_prompt'), 1, backward_compatibility, backcomp).
 index(('$strip_module'), 3, backward_compatibility, backcomp).
 index(('$declare_module'), 3, backward_compatibility, backcomp).
 index((at_initialization), 1, backward_compatibility, backcomp).
@@ -95,6 +102,8 @@ index((sformat), 3, backward_compatibility, backcomp).
 index((concat), 3, backward_compatibility, backcomp).
 index((concat_atom), 2, backward_compatibility, backcomp).
 index((concat_atom), 3, backward_compatibility, backcomp).
+index((read_clause), 1, backward_compatibility, backcomp).
+index((read_clause), 2, backward_compatibility, backcomp).
 index((read_variables), 2, backward_compatibility, backcomp).
 index((read_variables), 3, backward_compatibility, backcomp).
 index((feature), 2, backward_compatibility, backcomp).
@@ -109,6 +118,7 @@ index((subsumes), 2, backward_compatibility, backcomp).
 index((hash_term), 2, backward_compatibility, backcomp).
 index((checklist), 2, backward_compatibility, backcomp).
 index((sublist), 3, backward_compatibility, backcomp).
+index((sumlist), 2, backward_compatibility, backcomp).
 index((convert_time), 2, backward_compatibility, backcomp).
 index((convert_time), 8, backward_compatibility, backcomp).
 index(('C'), 3, backward_compatibility, backcomp).
@@ -123,27 +133,13 @@ index((setup_and_call_cleanup), 3, backward_compatibility, backcomp).
 index((setup_and_call_cleanup), 4, backward_compatibility, backcomp).
 index((merge), 3, backward_compatibility, backcomp).
 index((merge_set), 3, backward_compatibility, backcomp).
+index((index), 1, backward_compatibility, backcomp).
+index((hash), 1, backward_compatibility, backcomp).
+index((set_base_module), 1, backward_compatibility, backcomp).
 index((base32), 2, base32, base32).
 index((base32), 3, base32, base32).
 index((base64), 2, base64, base64).
 index((base64), 3, base64, base64).
-index((please), 2, bim, bim).
-index((cputime), 1, bim, bim).
-index((include), 1, bim, bim).
-index((setdebug), 0, bim, bim).
-index((reconsult), 1, bim, bim).
-index((rerecord), 2, bim, bim).
-index((erase_all), 1, bim, bim).
-index((record), 3, bim, bim).
-index((inttoatom), 2, bim, bim).
-index((atomconcat), 3, bim, bim).
-index((update), 1, bim, bim).
-index((printf), 2, bim, bim).
-index((index), 2, bim, bim).
-index((predicate_type), 2, bim, bim).
-index((vread), 2, bim, bim).
-index((bindVariables), 1, bim, bim).
-index((writeClause), 2, bim, bim).
 index((listen), 3, broadcast, broadcast).
 index((listen), 2, broadcast, broadcast).
 index((unlisten), 1, broadcast, broadcast).
@@ -179,14 +175,28 @@ index((find_chr_constraint), 1, chr, chr).
 index((chr_trace), 0, chr, chr).
 index((chr_notrace), 0, chr, chr).
 index((chr_leash), 1, chr, chr).
+index((format_to_codes), 3, codesio, codesio).
+index((format_to_codes), 4, codesio, codesio).
+index((write_to_codes), 2, codesio, codesio).
+index((write_to_codes), 3, codesio, codesio).
+index((write_term_to_codes), 3, codesio, codesio).
+index((write_term_to_codes), 4, codesio, codesio).
+index((read_from_codes), 2, codesio, codesio).
+index((read_term_from_codes), 3, codesio, codesio).
+index((open_codes_stream), 2, codesio, codesio).
+index((with_output_to_codes), 2, codesio, codesio).
+index((with_output_to_codes), 3, codesio, codesio).
+index((with_output_to_codes), 4, codesio, codesio).
 index((coinductive), 1, coinduction, coinduction).
 index((crypt), 2, crypt, crypt).
 index((csv), 3, csv, csv).
 index((csv), 4, csv, csv).
 index((csv_read_file), 2, csv, csv).
 index((csv_read_file), 3, csv, csv).
+index((csv_read_file_row), 3, csv, csv).
 index((csv_write_file), 2, csv, csv).
 index((csv_write_file), 3, csv, csv).
+index((csv_write_stream), 3, csv, csv).
 index((is_alnum), 1, ctypes, ctypes).
 index((is_alpha), 1, ctypes, ctypes).
 index((is_ascii), 1, ctypes, ctypes).
@@ -211,10 +221,12 @@ index((is_upper), 1, ctypes, ctypes).
 index((is_white), 1, ctypes, ctypes).
 index((to_lower), 2, ctypes, ctypes).
 index((to_upper), 2, ctypes, ctypes).
+index((upper_lower), 2, ctypes, ctypes).
 index((date_time_value), 3, date, date).
 index((parse_time), 2, date, date).
 index((parse_time), 3, date, date).
 index((day_of_the_week), 2, date, date).
+index((day_of_the_year), 2, date, date).
 index((debug), 3, prolog_debug, debug).
 index((debug), 1, prolog_debug, debug).
 index((nodebug), 1, prolog_debug, debug).
@@ -240,21 +252,12 @@ index((double_metaphone), 3, double_metaphone, double_metaphone).
 index((display), 1, edinburgh, edinburgh).
 index((display), 2, edinburgh, edinburgh).
 index((unknown), 2, edinburgh, edinburgh).
+index((reconsult), 1, edinburgh, edinburgh).
 index((debug), 0, edinburgh, edinburgh).
 index((nodebug), 0, edinburgh, edinburgh).
 index((fileerrors), 2, edinburgh, edinburgh).
 index((edit), 1, prolog_edit, edit).
 index((edit), 0, prolog_edit, edit).
-index(('$editor_load_code'), 2, emacs_interface, emacs_interface).
-index((find_predicate1), 2, emacs_interface, emacs_interface).
-index((emacs_consult), 1, emacs_interface, emacs_interface).
-index((emacs_dabbrev_atom), 1, emacs_interface, emacs_interface).
-index((emacs_complete_atom), 1, emacs_interface, emacs_interface).
-index((emacs_previous_command), 0, emacs_interface, emacs_interface).
-index((emacs_next_command), 0, emacs_interface, emacs_interface).
-index((call_emacs), 1, emacs_interface, emacs_interface).
-index((call_emacs), 2, emacs_interface, emacs_interface).
-index((running_under_emacs_interface), 0, emacs_interface, emacs_interface).
 index((type_error), 2, error, error).
 index((domain_error), 2, error, error).
 index((existence_error), 2, error, error).
@@ -279,9 +282,25 @@ index((relative_file_name), 3, files_ex, filesex).
 index((directory_file_path), 3, files_ex, filesex).
 index((copy_file), 2, files_ex, filesex).
 index((make_directory_path), 1, files_ex, filesex).
+index((copy_directory), 2, files_ex, filesex).
+index((delete_directory_and_contents), 1, files_ex, filesex).
+index((delete_directory_contents), 1, files_ex, filesex).
 index((reset_gensym), 0, gensym, gensym).
 index((reset_gensym), 1, gensym, gensym).
 index((gensym), 2, gensym, gensym).
+index((git), 2, git, git).
+index((git_process_output), 3, git, git).
+index((git_open_file), 4, git, git).
+index((git_describe), 2, git, git).
+index((git_remote_url), 3, git, git).
+index((git_ls_remote), 3, git, git).
+index((git_remote_branches), 2, git, git).
+index((git_default_branch), 2, git, git).
+index((git_tags_on_branch), 3, git, git).
+index((git_shortlog), 3, git, git).
+index((git_log_data), 3, git, git).
+index((git_show), 4, git, git).
+index((git_commit_data), 3, git, git).
 index((add_to_heap), 4, heaps, heaps).
 index((delete_from_heap), 4, heaps, heaps).
 index((empty_heap), 1, heaps, heaps).
@@ -322,12 +341,17 @@ index((nextto), 3, lists, lists).
 index((delete), 3, lists, lists).
 index((nth0), 3, lists, lists).
 index((nth1), 3, lists, lists).
+index((nth0), 4, lists, lists).
+index((nth1), 4, lists, lists).
 index((last), 2, lists, lists).
+index((proper_length), 2, lists, lists).
 index((same_length), 2, lists, lists).
 index((reverse), 2, lists, lists).
 index((permutation), 2, lists, lists).
 index((flatten), 2, lists, lists).
-index((sumlist), 2, lists, lists).
+index((max_member), 2, lists, lists).
+index((min_member), 2, lists, lists).
+index((sum_list), 2, lists, lists).
 index((max_list), 2, lists, lists).
 index((min_list), 2, lists, lists).
 index((numlist), 3, lists, lists).
@@ -352,6 +376,7 @@ index((memory_file_to_atom), 3, memory_file, memfile).
 index((memory_file_to_codes), 3, memory_file, memfile).
 index((utf8_position_memory_file), 3, memory_file, memfile).
 index((mime_parse), 2, mime, mime).
+index((mime_default_charset), 2, mime, mime).
 index((nb_rb_insert), 3, nb_rbtrees, nb_rbtrees).
 index((nb_rb_get_node), 3, nb_rbtrees, nb_rbtrees).
 index((nb_rb_node_value), 2, nb_rbtrees, nb_rbtrees).
@@ -402,7 +427,6 @@ index((ord_subset), 2, ordsets, ordsets).
 index((ord_empty), 1, ordsets, ordsets).
 index((ord_memberchk), 2, ordsets, ordsets).
 index((ord_symdiff), 3, ordsets, ordsets).
-index((ord_member), 2, ordsets, ordsets).
 index((ord_seteq), 2, ordsets, ordsets).
 index((ord_intersection), 2, ordsets, ordsets).
 index((oset_is), 1, oset, oset).
@@ -420,6 +444,7 @@ index((pairs_keys), 2, pairs, pairs).
 index((group_pairs_by_key), 2, pairs, pairs).
 index((transpose_pairs), 2, pairs, pairs).
 index((map_list_to_pairs), 3, pairs, pairs).
+index((pdt_install_console), 0, pdt_console, pdt_console).
 index((persistent), 1, persistency, persistency).
 index((current_persistent_predicate), 1, persistency, persistency).
 index((db_attach), 2, persistency, persistency).
@@ -442,6 +467,18 @@ index((tokenize_atom), 2, porter_stem, porter_stem).
 index((atom_to_stem_list), 2, porter_stem, porter_stem).
 index((portray_text), 1, portray_text, portray_text).
 index((set_portray_text), 2, portray_text, portray_text).
+index((predicate_options), 3, predicate_options, predicate_options).
+index((assert_predicate_options), 4, predicate_options, predicate_options).
+index((current_option_arg), 2, predicate_options, predicate_options).
+index((current_predicate_option), 3, predicate_options, predicate_options).
+index((check_predicate_option), 3, predicate_options, predicate_options).
+index((current_predicate_options), 3, predicate_options, predicate_options).
+index((retractall_predicate_options), 0, predicate_options, predicate_options).
+index((derived_predicate_options), 3, predicate_options, predicate_options).
+index((derived_predicate_options), 1, predicate_options, predicate_options).
+index((check_predicate_options), 0, predicate_options, predicate_options).
+index((derive_predicate_options), 0, predicate_options, predicate_options).
+index((check_predicate_options), 1, predicate_options, predicate_options).
 index((process_create), 3, process, process).
 index((process_wait), 2, process, process).
 index((process_wait), 3, process, process).
@@ -451,12 +488,33 @@ index((is_process), 1, process, process).
 index((process_release), 1, process, process).
 index((process_kill), 1, process, process).
 index((process_kill), 2, process, process).
+index((autoload), 0, prolog_autoload, prolog_autoload).
+index((autoload), 1, prolog_autoload, prolog_autoload).
+index((set_breakpoint), 4, prolog_breakpoints, prolog_breakpoints).
+index((delete_breakpoint), 1, prolog_breakpoints, prolog_breakpoints).
+index((breakpoint_property), 2, prolog_breakpoints, prolog_breakpoints).
 index((clause_info), 4, prolog_clause, prolog_clause).
 index((predicate_name), 2, prolog_clause, prolog_clause).
 index((clause_name), 2, prolog_clause, prolog_clause).
+index((prolog_walk_code), 1, prolog_codewalk, prolog_codewalk).
+index((prolog_colourise_stream), 3, prolog_colour, prolog_colour).
+index((prolog_colourise_term), 4, prolog_colour, prolog_colour).
+index((syntax_colour), 2, prolog_colour, prolog_colour).
+index((syntax_message), 3, prolog_colour, prolog_colour).
 index((prolog_history), 1, prolog_history, prolog_history).
+index((pack_list_installed), 0, prolog_pack, prolog_pack).
+index((pack_info), 1, prolog_pack, prolog_pack).
+index((pack_list), 1, prolog_pack, prolog_pack).
+index((pack_search), 1, prolog_pack, prolog_pack).
+index((pack_install), 1, prolog_pack, prolog_pack).
+index((pack_install), 2, prolog_pack, prolog_pack).
+index((pack_upgrade), 1, prolog_pack, prolog_pack).
+index((pack_rebuild), 1, prolog_pack, prolog_pack).
+index((pack_rebuild), 0, prolog_pack, prolog_pack).
+index((pack_remove), 1, prolog_pack, prolog_pack).
 index((prolog_server), 2, prolog_server, prolog_server).
 index((prolog_read_source_term), 4, prolog_source, prolog_source).
+index((read_source_term_at_location), 3, prolog_source, prolog_source).
 index((prolog_open_source), 2, prolog_source, prolog_source).
 index((prolog_close_source), 1, prolog_source, prolog_source).
 index((prolog_canonical_source), 2, prolog_source, prolog_source).
@@ -465,7 +523,9 @@ index((file_alias_path), 2, prolog_source, prolog_source).
 index((path_segments_atom), 2, prolog_source, prolog_source).
 index((get_prolog_backtrace), 2, prolog_stack, prolog_stack).
 index((get_prolog_backtrace), 3, prolog_stack, prolog_stack).
+index((prolog_stack_frame_property), 2, prolog_stack, prolog_stack).
 index((print_prolog_backtrace), 2, prolog_stack, prolog_stack).
+index((print_prolog_backtrace), 3, prolog_stack, prolog_stack).
 index((backtrace), 1, prolog_stack, prolog_stack).
 index((xref_source), 1, prolog_xref, prolog_xref).
 index((xref_called), 3, prolog_xref, prolog_xref).
@@ -544,12 +604,19 @@ index((midstring), 6, quintus, quintus).
 index((raise_exception), 1, quintus, quintus).
 index((on_exception), 3, quintus, quintus).
 index((random), 1, random, random).
-index((random), 3, random, random).
-index((randseq), 3, random, random).
-index((randset), 3, random, random).
+index((random_between), 3, random, random).
 index((getrand), 1, random, random).
 index((setrand), 1, random, random).
+index((maybe), 0, random, random).
+index((maybe), 1, random, random).
+index((maybe), 2, random, random).
+index((random_perm2), 4, random, random).
+index((random_member), 2, random, random).
+index((random_select), 3, random, random).
+index((randseq), 3, random, random).
+index((randset), 3, random, random).
 index((random_permutation), 2, random, random).
+index((random), 3, random, random).
 index((rb_new), 1, rbtrees, rbtrees).
 index((rb_empty), 1, rbtrees, rbtrees).
 index((rb_lookup), 3, rbtrees, rbtrees).
@@ -665,6 +732,7 @@ index((sha_hash), 3, crypto_hash, sha).
 index((sha_new_ctx), 2, crypto_hash, sha).
 index((sha_hash_ctx), 4, crypto_hash, sha).
 index((hmac_sha), 4, crypto_hash, sha).
+index((hash_atom), 2, crypto_hash, sha).
 index((ls), 0, shell, shell).
 index((ls), 1, shell, shell).
 index((cd), 0, shell, shell).
@@ -719,6 +787,8 @@ index((ssl_open), 3, ssl, ssl).
 index((ssl_open), 4, ssl, ssl).
 index((ssl_negotiate), 5, ssl, ssl).
 index((ssl_exit), 1, ssl, ssl).
+index((ssl_session), 2, ssl, ssl).
+index((statistics), 0, prolog_statistics, statistics).
 index((time), 1, prolog_statistics, statistics).
 index((profile), 1, prolog_statistics, statistics).
 index((profile), 3, prolog_statistics, statistics).
@@ -752,6 +822,7 @@ index((compare_strings), 4, table, table).
 index((prefix_string), 3, table, table).
 index((prefix_string), 4, table, table).
 index((sub_string), 3, table, table).
+index((table_version), 2, table, table).
 index((sort_table), 2, table_util, table_util).
 index((verify_table_order), 1, table_util, table_util).
 index((term_hash), 2, terms, terms).
@@ -766,7 +837,6 @@ index((acyclic_term), 1, terms, terms).
 index((term_subsumer), 3, terms, terms).
 index((term_factorized), 3, terms, terms).
 index((show_coverage), 1, prolog_cover, test_cover).
-index((covered_clauses), 4, prolog_cover, test_cover).
 index((make_tests), 3, test_wizard, test_wizard).
 index((make_test), 3, test_wizard, test_wizard).
 index((concurrent), 3, thread, thread).
@@ -801,6 +871,9 @@ index((call_with_time_limit), 2, time, time).
 index((tty_clear), 0, tty, tty).
 index((tty_flash), 0, tty, tty).
 index((menu), 3, tty, tty).
+index((udp_host_to_address), 2, udp_broadcast, udp_broadcast).
+index((udp_broadcast_initialize), 2, udp_broadcast, udp_broadcast).
+index((udp_broadcast_service), 2, udp_broadcast, udp_broadcast).
 index((add_edges), 3, ugraphs, ugraphs).
 index((add_vertices), 3, ugraphs, ugraphs).
 index((complement), 2, ugraphs, ugraphs).
@@ -832,6 +905,12 @@ index((seteuid), 1, uid, uid).
 index((setegid), 1, uid, uid).
 index((set_user_and_group), 1, uid, uid).
 index((set_user_and_group), 2, uid, uid).
+index((unicode_property), 2, unicode, unicode).
+index((unicode_map), 3, unicode, unicode).
+index((unicode_nfd), 2, unicode, unicode).
+index((unicode_nfc), 2, unicode, unicode).
+index((unicode_nfkd), 2, unicode, unicode).
+index((unicode_nfkc), 2, unicode, unicode).
 index((fork), 1, unix, unix).
 index((exec), 1, unix, unix).
 index((wait), 2, unix, unix).
@@ -866,8 +945,16 @@ index((url_iri), 2, url, url).
 index((file_name_to_url), 2, url, url).
 index((set_url_encoding), 2, url, url).
 index((utf8_codes), 3, utf8, utf8).
+index((numbervars), 1, varnumbers, varnumbers).
+index((varnumbers), 2, varnumbers, varnumbers).
+index((max_var_number), 3, varnumbers, varnumbers).
+index((varnumbers), 3, varnumbers, varnumbers).
 index((vm_list), 1, prolog_vm, vm).
 index((when), 2, when, when).
+index((writef), 1, writef, writef).
+index((writef), 2, writef, writef).
+index((swritef), 2, writef, writef).
+index((swritef), 3, writef, writef).
 index((www_open_url), 1, www_browser, www_browser).
 index((expand_url_path), 2, www_browser, www_browser).
 index((xpath), 3, xpath, xpath).
