@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -40,9 +40,8 @@
 :- use_module(library(http/http_parameters)).
 :- use_module(library(http/html_write)).
 :- use_module(library(http/mimetype)).
-:- use_module(library(http/dcg_basics)).
+:- use_module(library(dcg/basics)).
 :- use_module(library(http/http_dispatch)).
-:- use_module(library(http/http_log)).
 :- use_module(library(http/http_hook)).
 :- use_module(library(http/http_path)).
 :- use_module(library(http/http_wrapper)).
@@ -408,6 +407,7 @@ file('private.png',   'private.png').
 file('reload.gif',    'reload.gif').
 file('favicon.ico',   'favicon.ico').
 file('h1-bg.png',     'h1-bg.png').
+file('h2-bg.png',     'h2-bg.png').
 file('pub-bg.png',    'pub-bg.png').
 file('priv-bg.png',   'priv-bg.png').
 file('multi-bg.png',  'multi-bg.png').
@@ -441,6 +441,10 @@ documentation(Path, Request) :-
 	file_base_name(Path, Base),
 	file(_, Base), !,			% serve pldoc.css, etc.
 	http_reply_file(pldoc(Base), [], Request).
+documentation(Path, Request) :-
+	file_name_extension(_, Ext, Path),
+	autolink_extension(Ext, image),
+	http_reply_file(Path, [unsafe(true)], Request).
 documentation(Path, Request) :-
 	Index = '/index.html',
 	sub_atom(Path, _, _, 0, Index),
@@ -451,13 +455,14 @@ documentation(Path, Request) :-
 	    doc_for_dir(Dir, EditOptions)
 	;   throw(http_reply(forbidden(Dir)))
 	).
-documentation(File, _Request) :-
+documentation(File, Request) :-
 	wiki_file(File, WikiFile), !,
 	(   allowed_file(WikiFile)
 	->  true
 	;   throw(http_reply(forbidden(File)))
 	),
-	doc_for_wiki_file(WikiFile, []).
+	edit_options(Request, Options),
+	doc_for_wiki_file(WikiFile, Options).
 documentation(Path, Request) :-
 	http_parameters(Request,
 			[ public_only(Public),
@@ -471,6 +476,10 @@ documentation(Path, Request) :-
 	->  true
 	;   throw(http_reply(forbidden(File)))
 	),
+        (   exists_file(File)
+        ->  true
+        ;   throw(http_reply(not_found(File)))
+        ),
 	(   Reload == true,
 	    source_file(File)
 	->  load_files(File, [if(changed), imports([])])
@@ -494,6 +503,8 @@ documentation(Path, Request) :-
 	).
 
 
+:- public src_skin/4.			% called through source_to_html/3.
+
 src_skin(Request, _Show, header, Out) :- !,
 	memberchk(request_uri(ReqURI), Request), !,
 	replace_parameters(ReqURI, [show(raw)], RawLink),
@@ -531,10 +542,10 @@ edit_options(_, []).
 
 pl_file(File, PlFile) :-
 	file_name_extension(Base, html, File), !,
-	absolute_file_name(Base,
+	absolute_file_name(Base, PlFile,
 			   [ file_type(prolog),
 			     access(read)
-			   ], PlFile).
+			   ]).
 pl_file(File, File).
 
 %%	wiki_file(+File, -TxtFile) is semidet.
@@ -572,17 +583,24 @@ clean_path(Path, Path).
 %
 %	    * predicate=PI
 %	    providing documentation from the manual on the predicate PI.
+%	    * function=PI
+%	    providing documentation from the manual on the function PI.
 %	    * 'CAPI'=F
 %	    providing documentation from the manual on the C-function F.
 
 pldoc_man(Request) :-
 	http_parameters(Request,
 			[ predicate(PI, [optional(true)]),
+			  function(Fun, [optional(true)]),
 			  'CAPI'(F,     [optional(true)])
 			]),
 	format(string(Title), 'Manual -- ~w', [PI]),
 	(   ground(PI)
 	->  Obj = PI
+	;   ground(Fun)
+	->  atomic_list_concat([Name,ArityAtom], /, Fun),
+	    atom_number(ArityAtom, Arity),
+	    Obj = f(Name/Arity)
 	;   ground(F)
 	->  Obj = c(F)
 	),
