@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -78,6 +78,27 @@ the same paradigm. The module can
 @author Jan Wielemaker
 */
 
+:- predicate_options(doc_latex/3, 3,
+		     [ stand_alone(boolean),
+		       public_only(boolean),
+		       section_level(oneof([section,subsection,subsubsection])),
+		       summary(atom)
+		     ]).
+:- predicate_options(latex_for_file/3, 3,
+		     [ stand_alone(boolean),
+		       public_only(boolean),
+		       section_level(oneof([section,subsection,subsubsection]))
+		     ]).
+:- predicate_options(latex_for_predicates/3, 3,
+		     [				% no options
+		     ]).
+:- predicate_options(latex_for_wiki_file/3, 3,
+		     [ stand_alone(boolean),
+		       public_only(boolean),
+		       section_level(oneof([section,subsection,subsubsection]))
+		     ]).
+
+
 :- thread_local
 	options/1,
 	documented/1.
@@ -283,12 +304,14 @@ latex([]) --> !,
 	[].
 latex(Atomic) -->
 	{ string(Atomic),
-	  sub_atom(Atomic, 0, _, 0, 'LaTeX')
+	  string_to_atom(Atomic, Atom),
+	  sub_atom(Atom, 0, _, 0, 'LaTeX')
 	}, !,
 	[ latex('\\LaTeX{}') ].
 latex(Atomic) -->			% can this actually happen?
 	{ atomic(Atomic), !,
-	  findall(x, sub_atom(Atomic, _, _, _, '\n'), Xs),
+	  string_to_atom(Atomic, Atom),
+	  findall(x, sub_atom(Atom, _, _, _, '\n'), Xs),
 	  length(Xs, Lines)
 	},
 	(   {Lines == 0}
@@ -417,7 +440,7 @@ indent(prefixop) --> !,	       [ nl(1), indent(4) ].
 indent(postfixop) --> !,       [ nl(1), indent(4) ].
 indent(predicatesummary) --> !,[ nl(1) ].
 indent(oppredsummary) --> !,   [ nl(1) ].
-indent(hline) --> !, 	       [ nl(1) ].
+indent(hline) --> !,	       [ nl(1) ].
 indent(_) -->		       [].
 
 outdent(begin) --> !,		[ nl_exact(1) ].
@@ -439,7 +462,7 @@ outdent(prefixop) --> !,	[ nl(1) ].
 outdent(postfixop) --> !,	[ nl(1) ].
 outdent(predicatesummary) --> !,[ nl(1) ].
 outdent(oppredsummary) --> !,   [ nl(1) ].
-outdent(hline) --> !, 	        [ nl(1) ].
+outdent(hline) --> !,	        [ nl(1) ].
 outdent(_) -->			[].
 
 %%	latex_special(String, Rest)// is semidet.
@@ -1295,7 +1318,8 @@ replace([H|T0], F, N, [H|T]) :-
 %	Print Text, such that it comes out as normal LaTeX text.
 
 print_latex(Out, String) :-
-	atom_chars(String, Chars),
+	string_to_atom(String, Atom),
+	atom_chars(Atom, Chars),
 	print_chars(Chars, Out).
 
 print_chars([], _).
