@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -34,6 +34,7 @@
 	    http_reply_file/3,		% +File, +Options, +Request
 	    http_redirect/3,		% +How, +Path, +Request
 	    http_404/2,			% +Options, +Request
+	    http_switch_protocol/2,	% :Goal, +Options
 	    http_current_handler/2,	% ?Path, ?Pred
 	    http_current_handler/3,	% ?Path, ?Pred
 	    http_location_by_id/2,	% +ID, -Location
@@ -52,6 +53,15 @@
 :- use_module(library(http/http_header)).
 :- use_module(library(http/thread_httpd)).
 
+:- predicate_options(http_404/2, 1, [index(any)]).
+:- predicate_options(http_reply_file/3, 2,
+		     [ cache(boolean),
+		       mime_type(any),
+		       pass_to(http_safe_file/2, 2)
+		     ]).
+:- predicate_options(http_safe_file/2, 2, [unsafe(boolean)]).
+:- predicate_options(http_switch_protocol/2, 2, []).
+
 /** <module> Dispatch requests in the HTTP server
 
 This module can be placed between   http_wrapper.pl  and the application
@@ -158,7 +168,8 @@ write_index(Request) :-
 :- meta_predicate
 	http_handler(+, :, +),
 	http_current_handler(?, :),
-	http_current_handler(?, :, ?).
+	http_current_handler(?, :, ?),
+	http_switch_protocol(2, +).
 
 http_handler(Path, Pred, Options) :-
 	strip_module(Pred, M, P),
@@ -739,6 +750,37 @@ http_404(_Options, Request) :-
 	throw(http_reply(not_found(Path))).
 
 
+%%	http_switch_protocol(:Goal, +Options)
+%
+%	Send an =|HTTP 101 Switching   Protocols"|= reply. After sending
+%	the  reply,  the  HTTP  library    calls   call(Goal,  InStream,
+%	OutStream), where InStream and OutStream are  the raw streams to
+%	the HTTP client. This allows the communication to continue using
+%	an an alternative protocol.
+%
+%	If Goal fails or throws an exception,  the streams are closed by
+%	the server. Otherwise  Goal  is   responsible  for  closing  the
+%	streams. Note that  Goal  runs  in   the  HTTP  handler  thread.
+%	Typically, the handler should be   registered  using the =spawn=
+%	option if http_handler/3 or Goal   must  call thread_create/3 to
+%	allow the HTTP worker to return to the worker pool.
+%
+%	The streams use binary  (octet)  encoding   and  have  their I/O
+%	timeout set to the server  timeout   (default  60  seconds). The
+%	predicate set_stream/2 can  be  used   to  change  the encoding,
+%	change or cancel the timeout.
+%
+%	This predicate interacts with the server  library by throwing an
+%	exception.
+%
+%	@param	Options is reserved for future extensions.  It must be
+%		initialised to the empty list ([]).
+%	@throws	http_reply(switch_protocol(Goal, Options))
+
+http_switch_protocol(Goal, Options) :-
+	throw(http_reply(switch_protocol(Goal, Options))).
+
+
 		 /*******************************
 		 *	  PATH COMPILATION	*
 		 *******************************/
