@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -54,6 +54,8 @@
 :- use_module(library(broadcast)).
 :- use_module(library(lists)).
 
+:- predicate_options(http_open_session/2, 2, [renew(boolean)]).
+
 /** <module> HTTP Session management
 
 This library defines session management based   on HTTP cookies. Session
