@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -34,6 +34,9 @@
 	  [ make/0
 	  ]).
 :- use_module(library(check)).
+:- use_module(library(lists)).
+:- use_module(library(option)).
+:- use_module(library(debug)).
 :- set_prolog_flag(generate_debug_info, false).
 
 /** <module>  Reload modified source files
@@ -60,14 +63,15 @@ make_no_trace :-
 	'$update_library_index',
 	findall(File, modified_file(File), Reload),
 	print_message(silent, make(reload(Reload))),
-	reload(Reload),
+	maplist(reload_file, Reload),
 	print_message(silent, make(done(Reload))),
 	list_undefined([scan(local)]).
 
 modified_file(File) :-
-	'$time_source_file'(Source, Time, user),
+	source_file_property(Source, modified(Time)),
+	\+ source_file_property(Source, included_in(_,_)),
 	Time > 0.0,			% See source_file/1
-	(   '$derived_source'(Source, File, LoadTime)
+	(   source_file_property(Source, derived_from(File, LoadTime))
 	->  true
 	;   File = Source,
 	    LoadTime = Time
@@ -75,34 +79,37 @@ modified_file(File) :-
 	(   catch(time_file(File, Modified), _, fail),
 	    Modified > LoadTime
 	->  true
-	;   system:'$included'(File, Included, InclLoadTime),
+	;   source_file_property(Source, includes(Included, IncLoadTime)),
 	    catch(time_file(Included, Modified), _, fail),
-	    Modified > InclLoadTime
+	    Modified > IncLoadTime
 	->  true
 	).
 
 
-reload([]).
-reload([H|T]) :-
-	reload_file(H),
-	reload(T).
-
 %%	reload_file(File)
 %
 %	Reload file into the proper module.
 %
-%	@bug	If the module was loaded using use_module/2, importing only
-%		some of the predicates, this is not know.
 %	@bug	If modules import each other, we must load them in the
 %		proper order for import/export dependencies.
 
+:- public reload_file/1.		% Used by PDT
+
 reload_file(File) :-
 	source_base_name(File, Compile),
-	findall(Context, system:'$load_context_module'(File, Context), Modules),
-	(   Modules = []
-	->  load_files(user:Compile)
-	;   forall('$member'(Context, Modules),
-		   load_files(Context:Compile))
+	findall(M-Opts,
+		source_file_property(File, load_context(M, _, Opts)),
+		Modules),
+	(   Modules = [First-OptsFirst|Rest]
+	->  merge_options([if(true), silent(false)], OptsFirst, OFirst),
+	    debug(make, 'Make: First load ~q', [load_files(First:Compile, OFirst)]),
+	    load_files(First:Compile, OFirst),
+	    forall(member(Context-Opts, Rest),
+		   ( merge_options([if(not_loaded), silent(false)], Opts, O),
+		     debug(make, 'Make: re-import: ~q', [load_files(Context:Compile, O)]),
+		     load_files(Context:Compile, O)
+		   ))
+	;   load_files(user:Compile)
 	).
 
 source_base_name(File, Compile) :-
