@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -42,6 +42,10 @@
 :- use_module(library(error)).
 :- use_module(library(aggregate)).
 
+:- predicate_options(db_attach/2, 2,
+		     [ sync(oneof([close,flush,none]))
+		     ]).
+
 /** <module> Provide persistent dynamic predicates
 
 This module provides simple persistent storage   for one or more dynamic
@@ -65,7 +69,7 @@ Below is a simple example:
 ==
 :- module(user_db,
 	  [ attach_user_db/1,		% +File
-	    user_role/2,		% ?User, ?Role
+	    current_user_role/2,	% ?User, ?Role
 	    add_user/2,			% +User, +Role
 	    set_user_role/2		% +User, +Role
 	  ]).
@@ -76,9 +80,9 @@ Below is a simple example:
 attach_user_db(File) :-
 	db_attach(File, []).
 
-%%	user_role(+Name, -Role) is semidet.
+%%	current_user_role(+Name, -Role) is semidet.
 
-user_role(Name, Role) :-
+current_user_role(Name, Role) :-
 	with_mutex(user_db, user_role(Name, Role)).
 
 add_user(Name, Role) :-
@@ -88,7 +92,7 @@ set_user_role(Name, Role) :-
 	user_role(Name, Role), !.
 set_user_role(Name, Role) :-
 	with_mutex(user_db,
-		   (  retractall_user_rule(Name, _),
+		   (  retractall_user_role(Name, _),
 		      assert_user_role(Name, Role))).
 ==
 
