@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -82,6 +82,7 @@
 :- use_module(library(debug)).
 :- use_module(library(apply)).
 :- use_module(library(pairs)).
+:- use_module(library(filesex)).
 :- use_module(doc_process).
 :- use_module(doc_man).
 :- use_module(doc_modes).
@@ -106,6 +107,76 @@ extracting module doc_wiki.pl into HTML+CSS.
 	tag//2.
 
 
+:- predicate_options(doc_for_wiki_file/2, 2,
+		     [ edit(boolean)
+		     ]).
+:- predicate_options(doc_hide_private/3, 3,
+		     [module(atom), public(list), public_only(boolean)]).
+:- predicate_options(edit_button//2, 2,
+		     [ edit(boolean)
+		     ]).
+:- predicate_options(file//2, 2,
+		     [ label(any),
+		       absolute_path(atom),
+		       href(atom),
+		       map_extension(list),
+		       files(list),
+		       edit_handler(atom)
+		     ]).
+:- predicate_options(file_header//2, 2,
+		     [ edit(boolean),
+		       files(list),
+		       public_only(boolean)
+		     ]).
+:- predicate_options(include//3, 3,
+		     [ absolute_path(atom),
+		       class(atom),
+		       files(list),
+		       href(atom),
+		       label(any),
+		       map_extension(list)
+		     ]).
+:- predicate_options(object_edit_button//2, 2,
+		     [ edit(boolean),
+		       pass_to(pred_edit_button//2, 2)
+		     ]).
+:- predicate_options(object_page//2, 2,
+		     [ for(any),
+		       header(boolean),
+		       links(boolean),
+		       no_manual(boolean),
+		       search_in(oneof([all,app,man])),
+		       search_match(oneof([name,summary])),
+		       search_options(boolean)
+		     ]).
+:- predicate_options(object_ref//2, 2,
+		     [ files(list),
+		       qualify(boolean),
+		       secref_style(oneof([number,title,number_title]))
+		     ]).
+:- predicate_options(object_synopsis//2, 2,
+		     [ href(atom)
+		     ]).
+:- predicate_options(pred_dt//3, 3,
+		     [ edit(boolean)
+		     ]).
+:- predicate_options(pred_edit_button//2, 2,
+		     [ edit(boolean)
+		     ]).
+:- predicate_options(predref//2, 2,
+		     [ files(list),
+		       prefer(oneof([manual,app])),
+		       pass_to(object_ref/4, 2)
+		     ]).
+:- predicate_options(private/2, 2,
+		     [ module(atom),
+		       public(list)
+		     ]).
+:- predicate_options(source_button//2, 2,
+		     [ files(list)
+		     ]).
+
+
 		 /*******************************
 		 *	     RESOURCES		*
 		 *******************************/
@@ -188,11 +259,10 @@ prolog_file(FileSpec, Options) -->
 %	@param File	Prolog canonical filename
 
 doc_file_objects(FileSpec, File, Objects, FileOptions, Options) :-
-	absolute_file_name(FileSpec,
+	absolute_file_name(FileSpec, File,
 			   [ file_type(prolog),
 			     access(read)
-			   ],
-			   File),
+			   ]),
 	Pos = File:Line,
 	ensure_doc_objects(File),
 	findall(Line-doc(Obj,Pos,Comment),
@@ -703,10 +773,7 @@ object_synopsis(M:Name/Arity, Options) -->
 	->  synopsis([code([':- use_module(',a(href(HREF), '~q'-[Unquoted]),').'])|Extra])
 	;   synopsis([code(':- use_module(~q).'-[Unquoted])|Extra])
 	).
-object_synopsis(_:Name/Arity, _) -->
-	{ functor(Head, Name, Arity),
-	  current_arithmetic_function(Head)
-	},
+object_synopsis(f(_/_), _) -->
 	synopsis(span(class(function),
 		      [ 'Arithmetic function (see ',
 			\object_ref(is/2, []),
@@ -787,7 +854,7 @@ doc_page_dom(Title, Body, DOM) :-
 
 print_html_head(Out) :-
 	format(Out,
-	       '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" \
+	       '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" \c
 	       "http://www.w3.org/TR/html4/strict.dtd">~n', []).
 
 % Rendering rules
@@ -1481,19 +1548,6 @@ in_file(Module, Head, File) :-
 	source_file(Module:Head, File).
 
 
-%%	relative_file_name(+Path:atom, +RelTo:atom, -RelPath:atom) is det.
-%
-%	True when RelPath is a relative path to Path, relative to RelTo.
-%
-%	@tbd	move to library?
-
-relative_file_name(Path, RelTo, RelPath) :-
-        atomic_list_concat(PL, /, Path),
-        atomic_list_concat(RL, /, RelTo),
-        delete_common_prefix(PL, RL, PL1, PL2),
-        to_dot_dot(PL2, DotDot, PL1),
-        atomic_list_concat(DotDot, /, RelPath).
-
 delete_common_prefix([H|T01], [H|T02], T1, T2) :- !,
         delete_common_prefix(T01, T02, T1, T2).
 delete_common_prefix(T1, T2, T1, T2).
@@ -1527,6 +1581,9 @@ to_dot_dot([_|T0], ['..'|T], Tail) :-
 %          List of file(Name, Link) that specifies that we must
 %	   user Link for the given physical file Name.
 %
+%	   * edit_handler(+Id)
+%	   HTTP handler Id to call if the user clicks the edit button.
+%
 %	@tbd	Translation of files to HREFS is a mess.  How to relate
 %		these elegantly?
 
@@ -1671,6 +1728,12 @@ include(File, image, Options) -->
 			], Attrs)
 	},
 	html(img(Attrs)).
+include(File, wiki, _Options) -->	% [[file.txt]] is included
+	{ access_file(File, read), !,
+	  read_file_to_codes(File, String, []),
+	  wiki_codes_to_dom(String, [], DOM)
+	},
+	html(DOM).
 include(File, _Type, Options) -->
 	link_file(File, Options), !.
 include(File, _, _) -->
