@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -64,6 +64,10 @@ well formatted HTML documents.
 @license GPL
 */
 
+:- predicate_options(doc_file_name/3, 3,
+		     [ format(oneof([html,tex]))
+		     ]).
+
 %%	prolog:predicate_summary(+PI, -Summary) is semidet.
 %
 %	Provide    predicate    summaries    to     the    XPCE    class
@@ -137,22 +141,25 @@ is_structured_comment(Comment, Prefixes) :-
 	is_list(Comment), !,
 	phrase(structured_comment(Prefixes), Comment, _).
 is_structured_comment(Comment, Prefixes) :-
-	sub_string(Comment, 0, _, _, '%%'), !,
-	sub_atom(Comment, 2, 1, _, Space),
+	string_to_atom(Comment, CommentA),
+	sub_atom(CommentA, 0, _, _, '%%'), !,
+	sub_atom(CommentA, 2, 1, _, Space),
 	char_type(Space, space),
 	\+ blanks_to_nl(Comment),
-	\+ sub_string(Comment, 2, _, _, ' SWI '),	% HACK
-	\+ sub_string(Comment, 2, _, _, ' SICStus '),	% HACK
-	\+ sub_string(Comment, 2, _, _, ' Mats '),	% HACK
+	\+ sub_atom(CommentA, 2, _, _, ' SWI '),	% HACK
+	\+ sub_atom(CommentA, 2, _, _, ' SICStus '),	% HACK
+	\+ sub_atom(CommentA, 2, _, _, ' Mats '),	% HACK
 	Prefixes = ["%"].
 is_structured_comment(Comment, Prefixes) :-
-	sub_string(Comment, 0, _, _, '/**'), !,
-	sub_atom(Comment, 3, 1, _, Space),
+	string_to_atom(Comment, CommentA),
+	sub_atom(CommentA, 0, _, _, '/**'), !,
+	sub_atom(CommentA, 3, 1, _, Space),
 	char_type(Space, space),
 	Prefixes = ["/**", " *"].
 
 blanks_to_nl(Comment) :-
-	sub_atom(Comment, At, 1, _, Char),
+	string_to_atom(Comment, CommentA),
+	sub_atom(CommentA, At, 1, _, Char),
 	At >= 2,
 	(   char_type(Char, end_of_line)
 	->  !
@@ -213,8 +220,8 @@ contains(Haystack, Needle) :-
 %	@param Doc	the name of the file documenting Source.
 %	@param Options	Option list:
 %
-%			* format(-Format)
-%			Output format.  One of =html= or =latex=
+%			* format(+Format)
+%			Output format.  One of =html= or =tex=
 %
 %	@error	permission_error(overwrite, Source)
 
