@@ -1,14 +1,34 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi-prolog.org
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2006-2008, University of Amsterdam
+    Copyright (C): 2006-2012, University of Amsterdam
+			      VU University Amsterdam
 
-    This file is covered by the `The Artistic License', also in use by
-    Perl.  See http://www.perl.com/pub/a/language/misc/Artistic.html
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+    As a special exception, if you link this library with other files,
+    compiled with a Free Software compiler, to produce an executable, this
+    library does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
+
+    Alternatively, this program may be distributed under the Perl
+    Artistic License, version 2.0.
 */
 
 :- module(plunit,
@@ -29,9 +49,13 @@ Unit testing environment for SWI-Prolog and   SICStus Prolog. For usage,
 please visit http://www.swi-prolog.org/pldoc/package/plunit.html.
 
 @author		Jan Wielemaker
-@license	artistic
+@license	GPL+SWI-exception or Artistic 2.0
 */
 
+:- use_module(library(apply)).
+:- meta_predicate valid_options(+, 1).
+
+
 		 /*******************************
 		 *    CONDITIONAL COMPILATION	*
 		 *******************************/
@@ -76,7 +100,8 @@ user:term_expansion(In, Out) :-
 	prolog_load_context(module, plunit),
 	if_expansion(In, Out).
 
-swi     :- catch(current_prolog_flag(dialect, swi), _, fail).
+swi     :- catch(current_prolog_flag(dialect, swi), _, fail), !.
+swi     :- catch(current_prolog_flag(dialect, yap), _, fail).
 sicstus :- catch(current_prolog_flag(system_type, _), _, fail).
 
 
@@ -93,6 +118,12 @@ current_test_flag(Name, Value) :-
 
 set_test_flag(Name, Value) :-
 	create_prolog_flag(Name, Value, []).
+
+:- if(current_prolog_flag(dialect, yap)).
+
+'$set_predicate_attribute'(_, _, _).
+
+:- endif.
 :- endif.
 
 :- if(sicstus).
@@ -245,7 +276,7 @@ begin_tests(Unit, Name, File:Line, Options) :-
 	    assert(current_unit(Unit, Name, Context, Options))
 	),
 	'$set_source_module'(Old, Name),
-	'$declare_module'(Name, Context, File, Line, false),
+	'$declare_module'(Name, test, Context, File, Line, false),
 	discontiguous(Name:'unit test'/4),
 	'$set_predicate_attribute'(Name:'unit test'/4, trace, 0),
 	discontiguous(Name:'unit body'/2),
@@ -254,11 +285,6 @@ begin_tests(Unit, Name, File:_Line, _Options) :-
 	'$set_source_module'(Old, Old),
 	asserta(loading_unit(Unit, Name, File, Old)).
 
-set_import_modules(Module, Imports) :-
-	findall(I, import_module(Module, I), IL),
-	forall(member(I, IL), delete_import_module(Module, I)),
-	forall(member(I, Imports), add_import_module(Module, I, end)).
-
 :- else.
 
 % we cannot use discontiguous as a goal in SICStus Prolog.
@@ -503,6 +529,7 @@ test_set_option(sto(V)) :-
 :- thread_local
 	passed/5,			% Unit, Test, Line, Det, Time
 	failed/4,			% Unit, Test, Line, Reason
+	failed_assertion/7,		% Unit, Test, Line, ALoc, STO, Reason, Goal
 	blocked/4,			% Unit, Test, Line, Reason
 	sto/4,				% Unit, Test, Line, Results
 	fixme/5.			% Unit, Test, Line, Reason, Status
@@ -512,19 +539,39 @@ test_set_option(sto(V)) :-
 
 %%	run_tests is semidet.
 %%	run_tests(+TestSet) is semidet.
+%
+%	Run  tests  and  report  about    the   results.  The  predicate
+%	run_tests/0 runs all known  tests  that   are  not  blocked. The
+%	predicate run_tests/1 takes a  specification   of  tests to run.
+%	This  is  either  a  single   specification    or   a   list  of
+%	specifications. Each single specification is  either the name of
+%	a test-unit or a term <test-unit>:<test>, denoting a single test
+%	within a unit.
 
 run_tests :-
 	cleanup,
-	forall(current_test_set(Set),
-	       run_unit(Set)),
-	report,
-	cleanup_after_test.
+	setup_call_cleanup(
+	    setup_trap_assertions(Ref),
+	    ( forall(current_test_set(Set),
+		     run_unit(Set)),
+	      check_for_test_errors
+	    ),
+	    ( cleanup_trap_assertions(Ref),
+	      report,
+	      cleanup_after_test
+	    )).
 
 run_tests(Set) :-
 	cleanup,
-	run_unit(Set),
-	report,
-	cleanup_after_test.
+	setup_call_cleanup(
+	    setup_trap_assertions(Ref),
+	    ( run_unit(Set),
+	      check_for_test_errors
+	    ),
+	    ( cleanup_trap_assertions(Ref),
+	      report,
+	      cleanup_after_test
+	    )).
 
 run_unit([]) :- !.
 run_unit([H|T]) :- !,
@@ -571,6 +618,7 @@ cleanup :-
 	thread_self(Me),
 	retractall(passed(_, _, _, _, _)),
 	retractall(failed(_, _, _, _)),
+	retractall(failed_assertion(_, _, _, _, _, _, _)),
 	retractall(blocked(_, _, _, _)),
 	retractall(sto(_, _, _, _)),
 	retractall(fixme(_, _, _, _, _)),
@@ -659,6 +707,59 @@ unification_capability(_) :-
 :- endif.
 :- endif.
 
+		 /*******************************
+		 *	ASSERTION HANDLING	*
+		 *******************************/
+
+:- if(swi).
+
+:- dynamic prolog:assertion_failed/2.
+
+setup_trap_assertions(Ref) :-
+	asserta((prolog:assertion_failed(Reason, Goal) :-
+			test_assertion_failed(Reason, Goal)),
+		Ref).
+
+cleanup_trap_assertions(Ref) :-
+	erase(Ref).
+
+test_assertion_failed(Reason, Goal) :-
+	thread_self(Me),
+	running(Unit, Test, Line, STO, Me),
+	(   catch(get_prolog_backtrace(10, Stack), _, fail),
+	    assertion_location(Stack, AssertLoc)
+	->  true
+	;   AssertLoc = unknown
+	),
+	current_test_flag(test_options, Options),
+	report_failed_assertion(Unit, Test, Line, AssertLoc,
+				STO, Reason, Goal, Options),
+	assert_cyclic(failed_assertion(Unit, Test, Line, AssertLoc,
+				       STO, Reason, Goal)).
+
+assertion_location(Stack, File:Line) :-
+	append(_, [AssertFrame,CallerFrame|_], Stack),
+	prolog_stack_frame_property(AssertFrame,
+				    predicate(prolog_debug:assertion/1)), !,
+	prolog_stack_frame_property(CallerFrame, location(File:Line)).
+
+report_failed_assertion(Unit, Test, Line, AssertLoc,
+			STO, Reason, Goal, _Options) :-
+	print_message(
+	    error,
+	    plunit(failed_assertion(Unit, Test, Line, AssertLoc,
+				    STO, Reason, Goal))).
+
+:- else.
+
+setup_trap_assertions(_).
+cleanup_trap_assertions(_).
+
+:- endif.
+
+
+
+
 
 		 /*******************************
 		 *	   RUNNING A TEST	*
@@ -680,7 +781,10 @@ run_test(Unit, Name, Line, Options, Body) :-
 run_test_once(Unit, Name, Line, Options, Body) :-
 	current_test_flag(test_options, GlobalOptions),
 	option(sto(false), GlobalOptions, false), !,
+	current_unification_capability(Type),
+	begin_test(Unit, Name, Line, Type),
 	run_test_6(Unit, Name, Line, Options, Body, Result),
+	end_test(Unit, Name, Line, Type),
 	report_result(Result, Options).
 run_test_once(Unit, Name, Line, Options, Body) :-
 	current_unit(Unit, _Module, _Supers, UnitOptions),
@@ -698,7 +802,9 @@ run_test_cap(Unit, Name, Line, Options, Body) :-
 	(   option(sto(Type), Options)
 	->  unification_capability(Type),
 	    set_unification_capability(Type),
+	    begin_test(Unit, Name, Line, Type),
 	    run_test_6(Unit, Name, Line, Options, Body, Result),
+	    end_test(Unit, Name, Line, Type),
 	    report_result(Result, Options)
 	;   findall(Key-(Type+Result),
 		    test_caps(Type, Unit, Name, Line, Options, Body, Result, Key),
@@ -852,7 +958,7 @@ nondet_test(Expected, Unit, Name, Line, Options, Body, Result) :-
 		    Time is (T1 - T0)/1000.0,
 		    (   nondet_compare(Expected, Bindings, Unit, Name, Line)
 		    ->  Result = success(Unit, Name, Line, true, Time)
-		    ;   Result = failure(Unit, Name, Line, wrong_answer)
+		    ;   Result = failure(Unit, Name, Line, wrong_answer(Expected, Bindings))
 		    ),
 		    cleanup(Module, Options)
 		;   Result = failure(Unit, Name, Line, E),
@@ -1093,24 +1199,36 @@ running_tests(Running) :-
 		), Running).
 
 
-%%	report is semidet.
+%%	check_for_test_errors is semidet.
 %
-%	True if there are no errors.  If errors were encountered, report
-%	them to current output and fail.
+%	True if there are no errors, otherwise false.
+
+check_for_test_errors :-
+	number_of_clauses(failed/4, Failed),
+	number_of_clauses(failed_assertion/7, FailedAssertion),
+	number_of_clauses(sto/4, STO),
+	Failed+FailedAssertion+STO =:= 0.     % fail on errors
+
+
+%%	report is det.
+%
+%	Print a summary of the tests that ran.
 
 report :-
 	number_of_clauses(passed/5, Passed),
 	number_of_clauses(failed/4, Failed),
+	number_of_clauses(failed_assertion/7, FailedAssertion),
 	number_of_clauses(blocked/4, Blocked),
 	number_of_clauses(sto/4, STO),
-	(   Passed+Failed+Blocked+STO =:= 0
+	(   Passed+Failed+FailedAssertion+Blocked+STO =:= 0
 	->  info(plunit(no_tests))
-	;   Failed+Blocked+STO =:= 0
+	;   Failed+FailedAssertion+Blocked+STO =:= 0
 	->  report_fixme,
 	    info(plunit(all_passed(Passed)))
 	;   report_blocked,
 	    report_fixme,
 	    report_failed,
+	    report_failed_assertions,
 	    report_sto
 	).
 
@@ -1136,19 +1254,15 @@ report_blocked.
 
 report_failed :-
 	number_of_clauses(failed/4, N),
-	N > 0, !,
-	info(plunit(failed(N))),
-	fail.
-report_failed :-
-	info(plunit(failed(0))).
+	info(plunit(failed(N))).
+
+report_failed_assertions :-
+	number_of_clauses(failed_assertion/7, N),
+	info(plunit(failed_assertions(N))).
 
 report_sto :-
 	number_of_clauses(sto/4, N),
-	N > 0, !,
-	info(plunit(sto(N))),
-	fail.
-report_sto :-
-	info(plunit(sto(0))).
+	info(plunit(sto(N))).
 
 report_fixme :-
 	report_fixme(_,_,_).
@@ -1264,6 +1378,9 @@ locationprefix(unit(Unit)) --> !,
 locationprefix(FileLine) -->
 	{ throw_error(type_error(locationprefix,FileLine), _) }.
 
+:- discontiguous
+	message//1.
+
 message(error(context_error(plunit_close(Name, -)), _)) -->
 	[ 'PL-Unit: cannot close unit ~w: no open unit'-[Name] ].
 message(error(context_error(plunit_close(Name, Start)), _)) -->
@@ -1321,6 +1438,12 @@ message(plunit(failed(1))) --> !,
 	[ '1 test failed'-[] ].
 message(plunit(failed(N))) -->
 	[ '~D tests failed'-[N] ].
+message(plunit(failed_assertions(0))) --> !,
+	[].
+message(plunit(failed_assertions(1))) --> !,
+	[ '1 assertion failed'-[] ].
+message(plunit(failed_assertions(N))) -->
+	[ '~D assertions failed'-[N] ].
 message(plunit(sto(0))) --> !,
 	[].
 message(plunit(sto(N))) -->
@@ -1333,13 +1456,55 @@ message(plunit(fixme(Failed,Passed,0))) -->
 	[ 'FIXME: ~D failed; ~D passed'-[Failed, Passed] ].
 message(plunit(fixme(Failed,Passed,Nondet))) -->
 	{ TotalPassed is Passed+Nondet },
-	[ 'FIXME: ~D failed; ~D passed; (~D nondet)'-[Failed, TotalPassed, Nondet] ].
+	[ 'FIXME: ~D failed; ~D passed; (~D nondet)'-
+	  [Failed, TotalPassed, Nondet] ].
 message(plunit(failed(Unit, Name, Line, Failure))) -->
        { unit_file(Unit, File) },
        locationprefix(File:Line),
        test_name(Name),
        [': '-[] ],
        failure(Failure).
+:- if(swi).
+message(plunit(failed_assertion(Unit, Name, Line, AssertLoc,
+				_STO, Reason, Goal))) -->
+       { unit_file(Unit, File) },
+       locationprefix(File:Line),
+       test_name(Name),
+       [ ': assertion'-[] ],
+       assertion_location(AssertLoc, File),
+       assertion_reason(Reason), ['\n\t'],
+       assertion_goal(Unit, Goal).
+
+assertion_location(File:Line, File) -->
+	[ ' at line ~w'-[Line] ].
+assertion_location(File:Line, _) -->
+	[ ' at ~w:~w'-[File, Line] ].
+assertion_location(unknown, _) -->
+	[].
+
+assertion_reason(fail) --> !,
+	[ ' failed'-[] ].
+assertion_reason(Error) -->
+	{ message_to_string(Error, String) },
+	[ ' raised "~w"'-[String] ].
+
+assertion_goal(Unit, Goal) -->
+	{ unit_module(Unit, Module),
+	  unqualify(Goal, Module, Plain)
+	},
+	[ 'Assertion: ~p'-[Plain] ].
+
+unqualify(Var, _, Var) :-
+	var(Var), !.
+unqualify(M:Goal, Unit, Goal) :-
+	nonvar(M),
+	unit_module(Unit, M), !.
+unqualify(M:Goal, _, Goal) :-
+	callable(Goal),
+	predicate_property(M:Goal, imported_from(system)), !.
+unqualify(Goal, _, Goal).
+
+:- endif.
 					% Setup/condition errors
 message(plunit(error(Where, Context, Exception))) -->
 	locationprefix(Context),
@@ -1452,6 +1617,21 @@ failure(wrong_answer(Cmp)) -->
 	},
 	[ 'wrong answer (compared using ~w)'-[Op], nl ],
 	expected_got_ops_(Ex, A, OPS, Goals).
+failure(wrong_answer(CmpExpected, Bindings)) -->
+	{ (   CmpExpected = all(Cmp)
+	  ->  Cmp =.. [Op,_,Expected],
+	      Got = Bindings,
+	      Type = all
+	  ;   CmpExpected = set(Cmp),
+	      Cmp =.. [Op,_,Expected0],
+	      sort(Expected0, Expected),
+	      sort(Bindings, Got),
+	      Type = set
+	  )
+	},
+	[ 'wrong "~w" answer:'-[Type] ],
+	[ nl, '    Expected: ~q'-[Expected] ],
+	[ nl, '       Found: ~q'-[Got] ].
 :- if(swi).
 failure(cmp_error(_Cmp, Error)) -->
 	{ message_to_string(Error, Message) },
@@ -1513,7 +1693,7 @@ user:generate_message_hook(Message) -->
 	message(Message),
 	[nl].				% SICStus requires nl at the end
 
-%	user:message_hook(+Severity, +Message, +Lines) is semidet.
+%%	user:message_hook(+Severity, +Message, +Lines) is semidet.
 %
 %	Redefine printing some messages. It appears   SICStus has no way
 %	to get multiple messages at the same   line, so we roll our own.
