@@ -3,9 +3,10 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2005, University of Amsterdam
+    Copyright (C): 1985-2011, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -19,7 +20,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -39,24 +40,46 @@
 :- use_module(library(occurs), [sub_term/2]).
 :- use_module(library(debug)).
 :- use_module(library(listing)).
+:- use_module(library(prolog_source)).
+
+:- public				% called from library(trace/clause)
+	unify_term/2,
+	make_varnames/5,
+	do_make_varnames/3.
+
+:- multifile
+	make_varnames_hook/5.
+
+/** <module> Get detailed source-information about a clause
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This module started life as part of the   GUI tracer. As it is generally
 useful for debugging  purposes  it  has   moved  to  the  general Prolog
-library. Being only applicable to debugging   and  sitting very close to
-the kernel, do not rely too much that its functionality will be stable.
+library.
 
-The tracer library  tracer/clause.pl  adds   caching  and  dealing  with
+The tracer library library(trace/clause) adds   caching and dealing with
 dynamic predicates using listing to  XPCE   objects  to  this. Note that
-clause_info/4 as below can be pretty slow.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+clause_info/4 as below can be slow.
+*/
 
 %%	clause_info(+ClauseRef, -File, -TermPos, -VarNames)
 %
-%	Fetches source information for the given clause.
+%	Fetches source information for the  given   clause.  File is the
+%	file from which the clause  was   loaded.  TermPos describes the
+%	source layout in a format   compatible  to the subterm_positions
+%	option of read_term/2.  VarNames provides access to the variable
+%	allocation in a stack-frame.  See make_varnames/5 for details.
+%
+%	Note that positions are  _|character   positions|_,  i.e., _not_
+%	bytes. Line endings count as a   single character, regardless of
+%	whether the actual ending is =|\n|= or =|\r\n|_.
 
 clause_info(ClauseRef, File, TermPos, NameOffset) :-
-	debug(clause_info, 'clause_info(~w)... ', [ClauseRef]),
+	(   debugging(clause_info)
+	->  clause_name(ClauseRef, Name),
+	    debug(clause_info, 'clause_info(~w) (~w)... ',
+		  [ClauseRef, Name])
+	;   true
+	),
 	clause_property(ClauseRef, file(File)),
 	'$clause'(Head, Body, ClauseRef, VarOffset),
 	(   Body == true
@@ -88,9 +111,6 @@ clause_info(ClauseRef, File, TermPos, NameOffset) :-
 %	NOTE: Called directly from  library(trace/clause)   for  the GUI
 %	tracer.
 
-:- public
-	unify_term/2.
-
 unify_term(X, X) :- !.
 unify_term(X1, X2) :-
 	compound(X1),
@@ -126,61 +146,29 @@ unify_args(I, Arity, T1, T2) :-
 	unify_args(A, Arity, T1, T2).
 
 
-%	Must be a user-programmable hook!
-
-alternate_syntax(prolog, _,    true,
-			       true).
-alternate_syntax(pce_class, M, pce_expansion:push_compile_operators(M),
-			       pce_expansion:pop_compile_operators) :-
-	current_prolog_flag(xpce, true).
-alternate_syntax(system, _,    style_check(+dollar),
-			       style_check(-dollar)).
-
-system_module(system) :- !.
-system_module(Module) :-
-	sub_atom(Module, 0, _, _, $), !.
+%%	read_term_at_line(+File, +Line, +Module,
+%%			  -Clause, -TermPos, -VarNames) is semidet.
+%
+%	Read a term from File at Line.
 
 read_term_at_line(File, Line, Module, Clause, TermPos, VarNames) :-
+	setup_call_cleanup(
+	    '$push_input_context'(clause_info),
+	    read_term_at_line_2(File, Line, Module, Clause, TermPos, VarNames),
+	    '$pop_input_context').
+
+read_term_at_line_2(File, Line, Module, Clause, TermPos, VarNames) :-
 	catch(open(File, read, In), _, fail),
-	call_cleanup(read(Line, In, Module, Clause, TermPos, VarNames),
-		     close(In)).
-
-read(Line, Handle, Module, Clause, TermPos, VarNames) :-
-	seek_to_line(Handle, Line),
-	read(Handle, Module, Clause, TermPos, VarNames).
-
-%%	read(+Stream, +Module, -Clause, -TermPos, -VarNames)
-%
-%	Read clause from Stream at current position with unknown syntax.
-%	It returns the term read  at  that   position,  as  well  as the
-%	subterm position info and variable names.
-%
-%	NOTE: Called directly from  library(trace/clause)   for  the GUI
-%	tracer.
-
-read(Handle, Module, Clause, TermPos, VarNames) :-
-	(   system_module(Module)
-	->  Syntax = system
-	;   true
-	),
-	stream_property(Handle, position(Here)),
-	alternate_syntax(Syntax, Module, Setup, Restore),
-	peek_char(Handle, X),
-	debug(clause_info, 'Using syntax ~w (c=~w)', [Syntax, X]),
-	Setup,
-	catch(read_term(Handle, Clause,
-			[ subterm_positions(TermPos),
-			  variable_names(VarNames),
-			  module(Module)
-			]),
-	      Error,
-	      true),
-	Restore,
-	(   var(Error)
-	->  !
-	;   set_stream_position(Handle, Here),
-	    fail
-	).
+	set_stream(In, newline(detect)),
+	call_cleanup(
+	    read_source_term_at_location(
+		In, Clause,
+		[ line(Line),
+		  module(Module),
+		  subterm_positions(TermPos),
+		  variable_names(VarNames)
+		]),
+	    close(In)).
 
 
 %%	make_varnames(+ReadClause, +DecompiledClause,
@@ -198,13 +186,6 @@ read(Handle, Module, Clause, TermPos, VarNames) :-
 %
 %	@param Offsets	List of Offset=Var
 %	@param Names	List of Name=Var
-%
-%	@bug Called directly from library(trace/clause) for the GUI tracer.
-
-:- multifile make_varnames_hook/5.
-:- public
-	make_varnames/5,
-	do_make_varnames/3.		% allow usage from the hook.
 
 make_varnames(ReadClause, DecompiledClause, Offsets, Names, Term) :-
 	make_varnames_hook(ReadClause, DecompiledClause, Offsets, Names, Term), !.
@@ -281,6 +262,7 @@ unify_clause((Head :- Read),
 				 [ PH,
 				   term_position(0,0,0,0,[0-0,PB])
 				 ]).
+					% DCG rules
 unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 	Read = (_ --> List, _),
 	is_list(List),
@@ -294,11 +276,11 @@ unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 					     term_position(_,_,_,_,[_,BP])
 					   ]), !,
 	TermPos1 = term_position(F,T,FF,FT,[ HP, BP ]),
-	match_module(Compiled2, Compiled1, TermPos1, TermPos).
+	match_module(Compiled2, Compiled1, Module, TermPos1, TermPos).
 					% general term-expansion
 unify_clause(Read, Compiled1, Module, TermPos0, TermPos) :-
 	ci_expand(Read, Compiled2, Module),
-	match_module(Compiled2, Compiled1, TermPos0, TermPos).
+	match_module(Compiled2, Compiled1, Module, TermPos0, TermPos).
 					% I don't know ...
 unify_clause(_, _, _, _, _) :-
 	debug(clause_info, 'Could not unify clause', []),
@@ -309,16 +291,27 @@ unify_clause_head(H1, H2) :-
 	strip_module(H2, _, H).
 
 ci_expand(Read, Compiled, Module) :-
-	catch(setup_call_cleanup('$set_source_module'(Old, Module),
-				 expand_term(Read, Compiled),
-				 '$set_source_module'(_, Old)),
+	catch(setup_call_cleanup(
+		  ( set_xref_flag(OldXRef),
+		    '$set_source_module'(Old, Module)
+		  ),
+		  expand_term(Read, Compiled),
+		  ( '$set_source_module'(_, Old),
+		    set_prolog_flag(xref, OldXRef)
+		  )),
 	      E,
 	      expand_failed(E, Read)).
 
-match_module((H1 :- B1), (H2 :- B2), Pos0, Pos) :- !,
+set_xref_flag(Value) :-
+	current_prolog_flag(xref, Value), !,
+	set_prolog_flag(xref, true).
+set_xref_flag(false) :-
+	create_prolog_flag(xref, true, [type(boolean)]).
+
+match_module((H1 :- B1), (H2 :- B2), Module, Pos0, Pos) :- !,
 	unify_clause_head(H1, H2),
-	unify_body(B1, B2, Pos0, Pos).
-match_module(H1, H2, Pos, Pos) :-	% deal with facts
+	unify_body(B1, B2, Module, Pos0, Pos).
+match_module(H1, H2, _, Pos, Pos) :-	% deal with facts
 	unify_clause_head(H1, H2).
 
 %%	expand_failed(+Exception, +Term)
@@ -331,19 +324,19 @@ expand_failed(E, Read) :-
 	debug(clause_info, 'Term-expand ~p failed: ~w', [Read, Msg]),
 	fail.
 
-%%	unify_body(+Read, +Decompiled, +Pos0, -Pos)
+%%	unify_body(+Read, +Decompiled, +Module, +Pos0, -Pos)
 %
 %	Deal with translations implied by the compiler.  For example,
 %	compiling (a,b),c yields the same code as compiling a,b,c.
 %
 %	Pos0 and Pos still include the term-position of the head.
 
-unify_body(B, B, Pos, Pos) :-
+unify_body(B, B, _, Pos, Pos) :-
 	does_not_dcg_after_binding(B, Pos), !.
-unify_body(R, D,
+unify_body(R, D, Module,
 	   term_position(F,T,FF,FT,[HP,BP0]),
 	   term_position(F,T,FF,FT,[HP,BP])) :-
-	ubody(R, D, BP0, BP).
+	ubody(R, D, Module, BP0, BP).
 
 %%	does_not_dcg_after_binding(+ReadBody, +ReadPos) is semidet.
 %
@@ -367,69 +360,83 @@ a --> { x, y, z }.
     which the compiler creates "a(X,Y) :- x, y, z, X=Y".
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-%%	ubody(+Read, +Decompiled, +TermPosRead, -TermPosForDecompiled)
+%%	ubody(+Read, +Decompiled, +Module, +TermPosRead, -TermPosForDecompiled)
 %
 %	@param Read		Clause read _after_ expand_term/2
 %	@param Decompiled	Decompiled clause
+%	@param Module		Load module
 %	@param TermPosRead	Sub-term positions of source
 
-ubody(B, B, P, P) :-
+ubody(B, B, _, P, P) :-
 	does_not_dcg_after_binding(B, P), !.
-ubody(X, call(X),			% X = call(X)
-      From-To,
-      term_position(From, To, From, To, [From-To])) :- !.
-ubody(B0, B,
+ubody(M:R, D, M, term_position(_,_,_,_,[_,RP]), TPOut) :-
+	ubody(R, D, M, RP, TPOut).
+ubody(X, call(X), _,			% X = call(X)
+      Pos,
+      term_position(From, To, From, To, [Pos])) :- !,
+	arg(1, Pos, From),
+	arg(2, Pos, To).
+ubody(B0, B, M,
       brace_term_position(F,T,A0),
       Pos) :-
 	B0 = (_,_=_), !,
 	T1 is T - 1,
-	ubody(B0, B,
+	ubody(B0, B, M,
 	      term_position(F,T,
 			    F,T,
 			    [A0,T1-T]),
 	      Pos).
-ubody(B0, B,
+ubody(B0, B, M,
       brace_term_position(F,T,A0),
       term_position(F,T,F,T,[A])) :- !,
-	ubody(B0, B, A0, A).
-ubody(C0, C, P0, P) :-
+	ubody(B0, B, M, A0, A).
+ubody(C0, C, M, P0, P) :-
 	nonvar(C0), nonvar(C),
 	C0 = (_,_), C = (_,_), !,
 	conj(C0, P0, GL, PL),
-	mkconj(C, P, GL, PL).
-ubody(X0, X,
+	mkconj(C, M, P, GL, PL).
+ubody(X0, X, M,
       term_position(F,T,FF,TT,PA0),
       term_position(F,T,FF,TT,PA)) :-
-	meta(X0), !,
+	meta(M, X0, S), !,
 	X0 =.. [_|A0],
 	X  =.. [_|A],
-	ubody_list(A0, A, PA0, PA).
+	S =.. [_|AS],
+	ubody_list(A0, A, AS, M, PA0, PA).
+ubody(X0, X, _,
+      term_position(F,T,FF,TT,PA0),
+      term_position(F,T,FF,TT,PA)) :-
+	expand_goal(X0, X, PA0, PA).
+
 					% 5.7.X optimizations
-ubody(_=_, true,			% singleton = Any
+ubody(_=_, true, _,			% singleton = Any
       term_position(F,T,_FF,_TT,_PA),
       F-T) :- !.
-ubody(_==_, fail,			% singleton/firstvar == Any
+ubody(_==_, fail, _,			% singleton/firstvar == Any
       term_position(F,T,_FF,_TT,_PA),
       F-T) :- !.
-ubody(A1=B1, B2=A2,			% Term = Var --> Var = Term
+ubody(A1=B1, B2=A2, _,			% Term = Var --> Var = Term
       term_position(F,T,FF,TT,[PA1,PA2]),
       term_position(F,T,FF,TT,[PA2,PA1])) :-
 	(A1==B1) =@= (B2==A2), !,
 	A1 = A2, B1=B2.
-ubody(A1==B1, B2==A2,			% const == Var --> Var == const
+ubody(A1==B1, B2==A2, _,		% const == Var --> Var == const
       term_position(F,T,FF,TT,[PA1,PA2]),
       term_position(F,T,FF,TT,[PA2,PA1])) :-
 	(A1==B1) =@= (B2==A2), !,
 	A1 = A2, B1=B2.
-ubody(A is B - C, A is B + C2, Pos, Pos) :-
+ubody(A is B - C, A is B + C2, _, Pos, Pos) :-
 	integer(C),
 	C2 =:= -C, !.
 
-ubody_list([], [], [], []).
-ubody_list([G0|T0], [G|T], [PA0|PAT0], [PA|PAT]) :-
-	ubody(G0, G, PA0, PA),
-	ubody_list(T0, T, PAT0, PAT).
+ubody_list([], [], [], _, [], []).
+ubody_list([G0|T0], [G|T], [AS|ASL], M, [PA0|PAT0], [PA|PAT]) :-
+	ubody_elem(AS, G0, G, M, PA0, PA),
+	ubody_list(T0, T, ASL, M, PAT0, PAT).
 
+ubody_elem(0, G0, G, M, PA0, PA) :- !,
+	ubody(G0, G, M, PA0, PA).
+ubody_elem(_, G, G, _, PA, PA).
 
 conj(Goal, Pos, GoalList, PosList) :-
 	conj(Goal, Pos, GoalList, [], PosList, []).
@@ -448,16 +455,16 @@ conj((!,(S=SR)), F-T, [!,S=SR|TG], TG, [F-T,F1-T1|TP], TP) :-
 conj(A, P, [A|TG], TG, [P|TP], TP).
 
 
-mkconj(Goal, Pos, GoalList, PosList) :-
-	mkconj(Goal, Pos, GoalList, [], PosList, []).
+mkconj(Goal, M, Pos, GoalList, PosList) :-
+	mkconj(Goal, M, Pos, GoalList, [], PosList, []).
 
-mkconj(Conj, term_position(0,0,0,0,[PA,PB]), GL, TG, PL, TP) :-
+mkconj(Conj, M, term_position(0,0,0,0,[PA,PB]), GL, TG, PL, TP) :-
 	nonvar(Conj),
 	Conj = (A,B), !,
-	mkconj(A, PA, GL, TGA, PL, TPA),
-	mkconj(B, PB, TGA, TG, TPA, TP).
-mkconj(A0, P0, [A|TG], TG, [P|TP], TP) :-
-	ubody(A, A0, P, P0).
+	mkconj(A, M, PA, GL, TGA, PL, TPA),
+	mkconj(B, M, PB, TGA, TG, TPA, TP).
+mkconj(A0, M, P0, [A|TG], TG, [P|TP], TP) :-
+	ubody(A, A0, M, P, P0).
 
 
 		 /*******************************
@@ -585,38 +592,36 @@ To keep track of the source-locations, we   have to redo the analysis of
 the clause as defined in init.pl
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+expand_goal(G, call(G), P, term_position(0,0,0,0,[P])) :-
+        var(G), !.
 expand_goal(G, G, P, P) :-
         var(G), !.
 expand_goal(M0, M, P0, P) :-
-	meta(M0), !,
+	meta(system, M0, S), !,
 	P0 = term_position(F,T,FF,FT,PL0),
 	P  = term_position(F,T,FF,FT,PL),
 	functor(M0, Functor, Arity),
 	functor(M,  Functor, Arity),
-	expand_meta_args(PL0, PL, 1, M0, M).
+	expand_meta_args(PL0, PL, 1, S, M0, M).
 expand_goal(A, B, P0, P) :-
         goal_expansion(A, B0, P0, P1), !,
 	expand_goal(B0, B, P1, P).
 expand_goal(A, A, P, P).
 
-expand_meta_args([], [], _, _, _).
-expand_meta_args([P0|T0], [P|T], I, M0, M) :-
+expand_meta_args([], [], _, _, _, _).
+expand_meta_args([P0|T0], [P|T], I, S, M0, M) :-
 	arg(I, M0, A0),
 	arg(I, M,  A),
-	expand_goal(A0, A, P0, P),
+	arg(I, S,  AS),
+	expand_arg(AS, A0, A, P0, P),
 	NI is I + 1,
-	expand_meta_args(T0, T, NI, M0, M).
+	expand_meta_args(T0, T, NI, S, M0, M).
 
-meta((_  ,  _)).
-meta((_  ;  _)).
-meta((_  -> _)).
-meta((_ *-> _)).
-meta((\+ _)).
-meta((not(_))).
-meta((call(_))).
-meta((once(_))).
-meta((ignore(_))).
-meta((forall(_, _))).
+expand_arg(0, A0, A, P0, P) :- !,
+	expand_goal(A0, A, P0, P).
+expand_arg(_, A, A, P, P).
+
+meta(M, G, S) :- predicate_property(M:G, meta_predicate(S)).
 
 goal_expansion(send(R, Msg), send_class(R, _, SuperMsg), P, P) :-
 	compound(Msg),
@@ -632,15 +637,27 @@ goal_expansion(SendSuperN, send_class(R, _, Msg), P, P) :-
 	compound(SendSuperN),
 	SendSuperN =.. [send_super, R, Sel | Args],
 	Msg =.. [Sel|Args].
+goal_expansion(SendN, send(R, Msg), P, P) :-
+	compound(SendN),
+	SendN =.. [send, R, Sel | Args],
+	atom(Sel), Args \== [],
+	Msg =.. [Sel|Args].
 goal_expansion(GetSuperN, get_class(R, _, Msg, Answer), P, P) :-
 	compound(GetSuperN),
 	GetSuperN =.. [get_super, R, Sel | AllArgs],
 	append(Args, [Answer], AllArgs),
 	Msg =.. [Sel|Args].
+goal_expansion(GetN, get(R, Msg, Answer), P, P) :-
+	compound(GetN),
+	GetN =.. [get, R, Sel | AllArgs],
+	append(Args, [Answer], AllArgs),
+	atom(Sel), Args \== [],
+	Msg =.. [Sel|Args].
 goal_expansion(G0, G, P, P) :-
 	user:goal_expansion(G0, G),	% TBD: we need the module!
 	G0 \== G.			% \=@=?
 
+
 		 /*******************************
 		 *	  PRINTABLE NAMES	*
 		 *******************************/
@@ -661,6 +678,10 @@ thaffix(1, st) :- !.
 thaffix(2, nd) :- !.
 thaffix(_, th).
 
+%%	predicate_name(:Head, -PredName:string) is det.
+%
+%	Describe a predicate as [Module:]Name/Arity.
+
 predicate_name(Predicate, PName) :-
 	strip_module(Predicate, Module, Head),
 	(   user:prolog_predicate_name(Module:Head, PName)
@@ -683,25 +704,8 @@ clause_name(Ref, Name) :-
 	predicate_name(Head, PredName),
 	thaffix(N, Th),
 	format(string(Name), '~d-~w clause of ~w', [N, Th, PredName]).
+clause_name(Ref, Name) :-
+	clause_property(Ref, erased), !,
+	clause_property(Ref, predicate(M:PI)),
+	format(string(Name), 'erased clause from ~q', [M:PI]).
 clause_name(_, '<meta-call>').
-
-
-		 /*******************************
-		 *        LOW-LEVEL STUFF	*
-		 *******************************/
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-These predicates communicate about lines.  We   should  consider using a
-line-cache for this for speed.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-%%	seek_to_line(+Stream, +Line)
-%
-%	Seek to indicated line-number.
-
-seek_to_line(Fd, N) :-
-	N > 1, !,
-	skip(Fd, 10),
-	NN is N - 1,
-	seek_to_line(Fd, NN).
-seek_to_line(_, _).
