@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -54,6 +54,11 @@ than compensated for by using block reads based on read_pending_input/3.
 @author Jan Wielemaker
 */
 
+:- predicate_options(phrase_from_file/3, 3,
+		     [ buffer_size(positive_integer),
+		       pass_to(system:open/4, 4)
+		     ]).
+
 %%	phrase_from_file(:Grammar, +File) is nondet.
 %
 %	Process the content of File  using   the  DCG  rule Grammar. The
@@ -82,8 +87,8 @@ than compensated for by using block reads based on read_pending_input/3.
 %	==
 
 :- meta_predicate
-	phrase_from_file(:, +),
-	phrase_from_file(:, +, +).
+	phrase_from_file(//, +),
+	phrase_from_file(//, +, +).
 
 phrase_from_file(Grammar, File) :-
 	phrase_from_file(Grammar, File, []).
@@ -142,19 +147,3 @@ read_to_input_stream(Stream, Pos, List) :-
 	;   read_pending_input(Stream, List, Tail),
 	    stream_to_lazy_list(Stream, Tail)
 	).
-
-
-		 /*******************************
-		 *	       IDE		*
-		 *******************************/
-
-% Using meta_predicate phrase_from_file(2, +)   is inappropriate because
-% the first argument is an  arbitrary  grammar.   With  :,  we make less
-% promises. We tell the IDE that we call   G+2. This is not complete and
-% should be fixed at some point.
-
-:- multifile
-	prolog:called_by/2.
-
-prolog:called_by(phrase_from_file(G, _File), [G+2]).
-prolog:called_by(phrase_from_file(G, _File, _Options), [G+2]).
