@@ -19,7 +19,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -53,7 +53,6 @@
 	  genarg/3,
 
 	  (mode)/1,
-	  (public)/1,
 	  no_style_check/1,
 	  otherwise/0,
 	  simple/1,
@@ -254,10 +253,9 @@ no_style_check(QOption) :-
 		*         DIRECTIVES            *
 		*********************************/
 
-% :- op(1150, fx, [(mode), (public)]).
+% :- op(1150, fx, [(mode)]).
 
 mode(_).
-public(_).
 
 
 		 /*******************************
