@@ -20,7 +20,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -153,6 +153,45 @@
 	rdf_save(+, :),
 	rdf_load(+, :).
 
+:- predicate_options(rdf_graph_prefixes/3, 3,
+		     [expand(callable), filter(callable), min_count(nonneg)]).
+:- predicate_options(rdf_load/2, 2,
+		     [ base_uri(atom),
+		       cache(boolean),
+		       db(atom),
+		       format(oneof([xml,triples,turtle])),
+		       graph(atom),
+		       if(oneof([true,changed,not_loaded])),
+		       modified(-float),
+		       silent(boolean),
+		       register_namespaces(boolean)
+		     ]).
+:- predicate_options(rdf_register_ns/3, 3, [force(boolean), keep(boolean)]).
+:- predicate_options(rdf_save/2, 2,
+		     [ graph(atom),
+		       db(atom),
+		       anon(boolean),
+		       base_uri(atom),
+		       write_xml_base(boolean),
+		       convert_typed_literal(callable),
+		       encoding(encoding),
+		       document_language(atom),
+		       namespaces(list(atom)),
+		       xml_attributes(boolean),
+		       inline(boolean)
+		     ]).
+:- predicate_options(rdf_save_header/2, 2,
+		     [ graph(atom),
+		       db(atom),
+		       namespaces(list(atom))
+		     ]).
+:- predicate_options(rdf_save_subject/3, 3,
+		     [ graph(atom),
+		       base_uri(atom),
+		       convert_typed_literal(callable),
+		       document_language(atom)
+		     ]).
+
 :- multifile
 	ns/2,
 	rdf_meta_specification/3.	% UnboundHead, Module, Head
@@ -220,13 +259,11 @@ rdf_register_ns(Alias, URI) :-
 rdf_register_ns(Alias, URI, _) :-
 	ns(Alias, URI), !.
 rdf_register_ns(Alias, URI, Options) :-
-	ns(Alias, _),
-	(   option(force(Force), Options, false),
-	    Force == true
+	ns(Alias, _), !,
+	(   option(force(true), Options, false)
 	->  retractall(ns(Alias, _)),
 	    assert(ns(Alias, URI))
-	;   option(keep(Keep), Options, false),
-	    Keep == true
+	;   option(keep(true), Options, false)
 	->  true
 	;   throw(error(permission_error(register, namespace, Alias),
 			context(_, 'Already defined')))
@@ -1444,7 +1481,8 @@ rdf_reset_db :-
 %			in a term stream(Out).
 
 :- thread_local
-	named_anon/2.			% +Resource, -Id
+	named_anon/2,			% +Resource, -Id
+	inlined/1.			% +Resource
 
 rdf_save(File) :-
 	rdf_save2(File, []).
@@ -1504,6 +1542,7 @@ cleanup_save(Reason,
 	flag(rdf_db_saved_subjects, SavedSubjects, OSavedSubjects),
 	flag(rdf_db_saved_triples, SavedTriples, OSavedTriples),
 	retractall(named_anon(_, _)),
+	retractall(inlined(_)),
 	(   Reason == exit
 	->  print_message(informational,
 			  rdf(saved(File, SavedSubjects, SavedTriples)))
@@ -1824,6 +1863,7 @@ xml_code(0'-).				% Match 0'-
 
 rdf_save_footer(Out) :-
 	retractall(named_anon(_, _)),
+	retractall(inlined(_)),
 	format(Out, '</rdf:RDF>~n', []).
 
 %%	rdf_save_non_anon_subject(+Out, +Subject, +Options)
@@ -1867,7 +1907,12 @@ rdf_save_subject(Out, Subject, DB) :-
 %
 %	@param Indent	Current indentation
 
+rdf_save_subject(_, Subject, _, _, _) :-
+	inlined(Subject), !.
 rdf_save_subject(Out, Subject, BaseURI, Indent, Options) :-
+	do_save_subject(Out, Subject, BaseURI, Indent, Options).
+
+do_save_subject(Out, Subject, BaseURI, Indent, Options) :-
 	graph(Options, DB),
 	findall(Pred=Object, rdf_db(Subject, Pred, Object, DB), Atts0),
 	sort(Atts0, Atts),		% remove duplicates
@@ -1929,14 +1974,14 @@ save_about(Out, BaseURI, Subject) :-
 	rdf_value(Subject, BaseURI, QSubject, Encoding),
 	format(Out, ' rdf:about="~w"', [QSubject]).
 
-%%	save_attributes(+List, +BaseURI, +Stream, Element)
+%%	save_attributes(+List, +BaseURI, +Stream, +Element, +Indent, +Options)
 %
 %	Save the attributes.  Short literal attributes are saved in the
 %	tag.  Others as the content of the description element.  The
 %	begin tag has already been filled.
 
 save_attributes(Atts, BaseURI, Out, Element, Indent, Options) :-
-	split_attributes(Atts, InTag, InBody),
+	split_attributes(Atts, InTag, InBody, Options),
 	SubIndent is Indent + 2,
 	save_attributes2(InTag, BaseURI, tag, Out, SubIndent, Options),
 	(   InBody == []
@@ -1948,13 +1993,15 @@ save_attributes(Atts, BaseURI, Out, Element, Indent, Options) :-
 	    format(Out, '>~n', [])
 	).
 
-%%	split_attributes(+Attributes, -HeadAttrs, -BodyAttr)
+%%	split_attributes(+Attributes, -HeadAttrs, -BodyAttr, Options)
 %
 %	Split attribute (Name=Value) list into attributes for the head
 %	and body. Attributes can only be in the head if they are literal
 %	and appear only one time in the attribute list.
 
-split_attributes(Atts, HeadAttr, BodyAttr) :-
+split_attributes(Atts, [], Atts, Options) :-
+	option(xml_attributes(false), Options), !.
+split_attributes(Atts, HeadAttr, BodyAttr, _) :-
 	duplicate_attributes(Atts, Dupls, Singles),
 	simple_literal_attributes(Singles, HeadAttr, Rest),
 	append(Dupls, Rest, BodyAttr).
@@ -2037,7 +2084,7 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
 		(S1 \== S2 ; Name \== P2)
 	    ->  predicate_property(named_anon(_,_), number_of_clauses(N)),
 		atom_concat('bn', N, NodeID),
-		assert(named_anon(Value, NodeID))
+		assertz(named_anon(Value, NodeID))
 	    ;	true
 	    ),
 	    SubIndent is Indent + 2,
@@ -2052,6 +2099,25 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
 	    rdf_write_id(Out, NameText),
 	    format(Out, '>~n', [])
 	).
+save_attribute(body, Name=Value, BaseURI, Out, Indent, Options) :-
+	option(inline(true), Options),
+	has_attributes(Value, Options),
+	\+ inlined(Value), !,
+	assertz(inlined(Value)),
+	rdf_id(Name, BaseURI, NameText),
+	format(Out, '~N~*|<', [Indent]),
+	rdf_write_id(Out, NameText),
+	SubIndent is Indent + 2,
+	(   rdf_collection(Value)
+	->  save_about(Out, BaseURI, Value),
+	    format(Out, ' rdf:parseType="Collection">~n', []),
+	    rdf_save_list(Out, Value, BaseURI, SubIndent, Options)
+	;   format(Out, '>~n', []),
+	    do_save_subject(Out, Value, BaseURI, SubIndent, Options)
+	),
+	format(Out, '~N~*|</', [Indent]),
+	rdf_write_id(Out, NameText),
+	format(Out, '>~n', []).
 save_attribute(body, Name=Value, BaseURI, Out, Indent, _DB) :-
 	stream_property(Out, encoding(Encoding)),
 	rdf_value(Value, BaseURI, QVal, Encoding),
@@ -2060,6 +2126,10 @@ save_attribute(body, Name=Value, BaseURI, Out, Indent, _DB) :-
 	rdf_write_id(Out, NameText),
 	format(Out, ' rdf:resource="~w"/>', [QVal]).
 
+has_attributes(URI, Options) :-
+	graph(Options, DB),
+	rdf_db(URI, _, _, DB), !.
+
 %%	save_body_literal(+Literal, +NameText, +BaseURI,
 %%			  +Out, +Indent, +Options).
 
@@ -2285,7 +2355,7 @@ rdf_quote_uri(IRI, URI) :-
 		 *******************************/
 
 :- multifile
-	prolog:message/3.
+	prolog:message//1.
 
 prolog:message(rdf(loaded(How, What, BaseURI, Triples, Time))) -->
 	how(How),
@@ -2316,9 +2386,9 @@ source(SourceURL) -->
 	{ uri_file_name(SourceURL, File), !,
 	  file_base_name(File, Base)	% TBD: relative file?
 	},
-	[ ' "~p"'-[Base] ].
+	[ ' "~w"'-[Base] ].
 source(SourceURL) -->
-	[ ' "~p"'-[SourceURL] ].
+	[ ' "~w"'-[SourceURL] ].
 
 into(_, _) --> [].			% TBD
 
