@@ -17,7 +17,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -237,10 +237,15 @@ rdf_save_canonical_turtle(Spec, M:Options) :-
 %	@param	Cleanup is a goal that must be used to revert the side
 %		effects of open_output/4.
 
-open_output(stream(Out), Encoding, Out,
-	    set_stream(Out, encoding(Old))) :- !,
+open_output(stream(Out), Encoding, Out, Cleanup) :- !,
 	stream_property(Out, encoding(Old)),
-	set_stream(Out, encoding(Encoding)).
+	(   (   Old == Encoding
+	    ;	Old == wchar_t		% Internal encoding
+	    )
+	->  Cleanup = true
+	;   set_stream(Out, encoding(Encoding)),
+	    Cleanup = set_stream(Out, encoding(Old))
+	).
 open_output(Stream, Encoding, Out, Cleanup) :-
 	\+ atom(Stream),
 	is_stream(Stream), !,
@@ -1115,9 +1120,6 @@ tw_literal(literal(lang(Lang, Value)), State, Out) :- !,
 	tw_quoted_string(Value, State, Out),
 	downcase_atom(Lang, TurtleLang),	% Turtle lang = [a-z]+('-'[a-z0-9]+)*
 	format(Out, '@~w', [TurtleLang]).
-tw_literal(literal(Value), State, Out) :-
-	atom(Value), !,
-	tw_quoted_string(Value, State, Out).
 tw_literal(literal(Value), State, Out) :-
 	atom(Value), !,
 	tw_quoted_string(Value, State, Out).
